// benchmark.ts benchmarks the proxy.  It should be running on localhost:9999
/// <reference path='../../../third_party/typings/es6-promise/es6-promise.d.ts' />
/// <reference path='../../../third_party/typings/request/request.d.ts' />
//// <reference path='../../../third_party/typings/node/node.d.ts' />
var request = require('request');
var util = require('util');
var shttpagent = require('socks5-http-client/lib/Agent');
var Benchmark;
(function (Benchmark) {
    var Bucket = (function () {
        function Bucket(up, cnt) {
            this.upperLimit = up;
            this.count = cnt;
        }
        return Bucket;
    })();
    Benchmark.Bucket = Bucket;
    ;
    var BasicStats = (function () {
        function BasicStats(values) {
            this.min = values[0];
            this.max = values[0];
            var sum = 0;
            var n = values.length;
            this.count = n;
            for (var i = 0; i < n; i++) {
                sum += values[i];
                if (values[i] < this.min) {
                    this.min = values[i];
                }
                if (values[i] > this.max) {
                    this.max = values[i];
                }
            }
            this.median = this.calcMedian(values);
            this.mean = sum / n;
        }
        BasicStats.prototype.calcMedian = function (values) {
            var sorted = values.sort();
            var len = values.length;
            if (len < 2) {
                return values[0];
            }
            if (len % 2) {
                // Odd number of elements, choose center.
                return sorted[Math.floor((len / 2) + 1)];
            }
            else {
                // Even number of elements, average two middle ones.
                return (sorted[len / 2] + sorted[(len / 2) + 1]) / 2;
            }
        };
        BasicStats.prototype.fmtNumber = function (n) {
            if (n) {
                return n.toFixed(2);
            }
            else {
                return "undef!";
            }
        };
        BasicStats.prototype.summary = function () {
            return util.format("[total: %d, min: %d, med: %d, mean: %d, max: %d]", this.count, this.min, this.fmtNumber(this.median), this.fmtNumber(this.mean), this.max);
        };
        return BasicStats;
    })();
    Benchmark.BasicStats = BasicStats;
    ;
    var Histogram = (function () {
        function Histogram(nbuckets, max) {
            this.buckets_ = new Array();
            this.count_ = 0;
            var step = max / nbuckets;
            for (var i = 0; i < nbuckets - 1; i++) {
                this.buckets_[i] = new Bucket(step * i, 0);
            }
            this.buckets_[nbuckets - 1] = new Bucket(Number.MAX_VALUE, 0);
        }
        Histogram.prototype.addValue = function (num) {
            for (var i = 0; i < this.buckets_.length; i++) {
                if (num < this.buckets_[i].upperLimit) {
                    this.buckets_[i].count++;
                    return;
                }
            }
        };
        Histogram.prototype.addValues = function (nums) {
            for (var i = 0; i < nums.length; i++) {
                this.addValue(nums[i]);
            }
            return this;
        };
        Histogram.prototype.getValues = function () {
            return this.buckets_;
        };
        Histogram.prototype.getPoints = function () {
            var buckets = new Array();
            for (var i = 0; i < this.buckets_.length; i++) {
                var prev = 0;
                if (i > 0) {
                    prev = this.buckets_[i - 1].upperLimit;
                }
                buckets.push([prev, this.buckets_[i].count]);
            }
            return buckets;
        };
        return Histogram;
    })();
    Benchmark.Histogram = Histogram;
    ;
    // Request result.  We separate out timeouts from the general
    // class of failure, as it probably indicates a bug in the SUT's
    // implementation.
    (function (Result) {
        Result[Result["RES_SUCCESS"] = 0] = "RES_SUCCESS";
        Result[Result["RES_FAILURE"] = 1] = "RES_FAILURE";
        Result[Result["RES_TIMEOUT"] = 2] = "RES_TIMEOUT";
    })(Benchmark.Result || (Benchmark.Result = {}));
    var Result = Benchmark.Result;
    ;
    // A container for raw latency values for a single kind of test.
    // Keeps separate value lists for each result.
    var DataVector = (function () {
        function DataVector() {
            this.values = [
                new Array(),
                new Array(),
                new Array(),
            ];
        }
        DataVector.prototype.addValue = function (latency, result) {
            this.values[result].push(latency);
        };
        DataVector.prototype.addValues = function (latencies, result) {
            for (var i = 0; i < latencies.length; i++) {
                this.values[result].push(latencies[i]);
            }
        };
        return DataVector;
    })();
    Benchmark.DataVector = DataVector;
    ;
    // A result for testing a single 'kind' of URL.
    var TestResult = (function () {
        function TestResult(size, successes, failures, timeouts, nbuckets, max) {
            this.requestSize = size;
            this.raw = new DataVector();
            this.histogram = new Array();
            var suc_hist = new Histogram(nbuckets, max);
            var fail_hist = new Histogram(nbuckets, max);
            var to_hist = new Histogram(nbuckets, max);
            suc_hist.addValues(successes);
            fail_hist.addValues(failures);
            to_hist.addValues(timeouts);
            this.histogram.push(suc_hist);
            this.histogram.push(fail_hist);
            this.histogram.push(to_hist);
            this.raw.addValues(successes, 0 /* RES_SUCCESS */);
            this.raw.addValues(failures, 1 /* RES_FAILURE */);
            this.raw.addValues(timeouts, 2 /* RES_TIMEOUT */);
        }
        return TestResult;
    })();
    Benchmark.TestResult = TestResult;
    ;
    // Data maintained about an in-flight HTTP request.
    var Request = (function () {
        function Request(url, sz, idx, num) {
            this.requestTime = Date.now();
            this.requestSizeIndex = idx;
            this.requestSize = sz;
            this.url = url;
            this.requestNum = num;
        }
        return Request;
    })();
    ;
    var RequestManager = (function () {
        function RequestManager(sizes) {
            // TODO: Size concurrency for underlying runtime.
            this.concurrency = 1;
            this.histoNumBuckets = 16;
            this.histoMax = 100;
            this.kTimeoutMS = 30000; // 30 sec timeout.
            this.kMaxTimeouts = 10; // max number of timeouts before
            // aborting.
            this.kWatchdogInterval = 500; // check 2 times/sec.
            this.latencies_ = new Array();
            this.sizes_ = sizes;
            this.request_queue_ = [];
            this.request_counter_ = 0;
            this.result_callback_ = null;
            this.running_requests_ = new Array();
            for (var i = 0; i < sizes.length; i++) {
                this.latencies_.push(new DataVector);
            }
        }
        RequestManager.prototype.configureDefaults = function (conc, nbuckets, max, verbosity, max_timeouts) {
            this.concurrency = conc;
            this.histoNumBuckets = nbuckets;
            this.histoMax = max;
            this.verbosity_ = verbosity;
            this.kMaxTimeouts = max_timeouts;
        };
        RequestManager.prototype.finishRequest = function (requestIndex, err, response, body) {
            var request_in_error = err != null;
            var req = this.running_requests_[requestIndex];
            if (req == null) {
                console.log("Getting a result back for a request that no longer " + "exists.  Race between timeouts?");
                return;
            }
            var result_time = Date.now();
            var latency_ms = result_time - req.requestTime;
            // first verify that the body is fully-formed
            if (!request_in_error && (!body || !body.length || body.length != req.requestSize)) {
                request_in_error = true;
            }
            if (req.requestTime < 0) {
                this.latencies_[req.requestSizeIndex].addValue(this.kTimeoutMS, 2 /* RES_TIMEOUT */);
            }
            else if (request_in_error) {
                // TODO: Look up error codes for this.
                this.latencies_[req.requestSizeIndex].addValue(latency_ms, 1 /* RES_FAILURE */);
                var body_length = -1;
                if (body) {
                    body_length = body.length;
                }
                console.log("--> finishRequest: got err: " + err + ", body length was " + body.length + ", wanted size: " + req.requestSize + ", on url " + req.url);
            }
            else {
                this.latencies_[req.requestSizeIndex].addValue(latency_ms, 0 /* RES_SUCCESS */);
                if (this.verbosity_ > 0) {
                    process.stdout.write("[" + latency_ms + " ms]\t");
                }
            }
            this.runATest(requestIndex);
        };
        RequestManager.prototype.startRequest = function (requestIndex, sizeIndex) {
            var size = this.sizes_[sizeIndex];
            var self = this;
            var url = 'http://localhost:8080/' + size;
            this.request_counter_++;
            this.running_requests_[requestIndex] = new Request(url, size, sizeIndex, this.request_counter_);
            request({
                url: 'http://localhost:8080/' + size,
                agent: new shttpagent({
                    socksHost: 'localhost',
                    socksPort: 9999
                })
            }, function (err, response, body) {
                self.finishRequest(requestIndex, err, response, body);
            });
        };
        RequestManager.prototype.runATest = function (requestIndex) {
            if (this.timeout_count_ > 0 && this.request_queue_.length > 0) {
                var queue_head = this.request_queue_[0];
                this.request_queue_.shift();
                this.startRequest(requestIndex, queue_head);
            }
            else if (this.finished_concurrent_requests_ < this.concurrency) {
                this.finished_concurrent_requests_++;
                this.running_requests_[requestIndex] = null;
            }
            if (this.finished_concurrent_requests_ == this.concurrency) {
                if (this.verbosity_ > 0) {
                    console.log("\nTest run complete.  Generating results");
                }
                var results = new Array();
                for (var sz = 0; sz < this.sizes_.length; sz++) {
                    results.push(new TestResult(this.sizes_[sz], this.latencies_[sz].values[0], this.latencies_[sz].values[1], this.latencies_[sz].values[2], this.histoNumBuckets, this.histoMax));
                }
                if (this.result_callback_ != null) {
                    var cb = this.result_callback_;
                    this.result_callback_ = null;
                    cb(results);
                }
            }
        };
        // Scan the running requests for anything that's timed out.
        RequestManager.prototype.watchDog = function () {
            var now = Date.now();
            var num_completed_requests = 0;
            for (var r = 0; r < this.running_requests_.length; r++) {
                var req = this.running_requests_[r];
                if (req != null) {
                    if (now - req.requestTime > this.kWatchdogInterval) {
                        req.requestTime = -1; // mark as timed out
                        console.log("*ruff! Timing out request " + req.requestNum + " on slot " + r);
                        this.finishRequest(r, null, null, null);
                        this.timeout_count_--;
                    }
                }
                else {
                    num_completed_requests++;
                }
            }
            // This setTimeout also keeps the node.js process running.
            if (!this.running_requests_.length || num_completed_requests < this.running_requests_.length) {
                var self = this;
                setTimeout(function () {
                    self.watchDog();
                }, this.kWatchdogInterval);
            }
        };
        RequestManager.prototype.initForTestRun = function () {
            this.timeout_count_ = this.kMaxTimeouts;
            this.running_requests_ = new Array();
            this.finished_concurrent_requests_ = 0;
            this.watchDog();
        };
        RequestManager.prototype.runTests = function (numPerSize, callback) {
            this.initForTestRun();
            this.result_callback_ = callback;
            for (var sz = 0; sz < this.sizes_.length; sz++) {
                for (var run = 0; run < numPerSize; run++) {
                    this.request_queue_.push(sz);
                }
            }
            for (var c = 0; c < this.concurrency; c++) {
                this.runATest(c);
            }
        };
        return RequestManager;
    })();
    Benchmark.RequestManager = RequestManager;
    ;
    ;
})(Benchmark = exports.Benchmark || (exports.Benchmark = {})); // module Benchmark
