/// <reference path='../../../../third_party/freedom-typings/freedom-core-env.d.ts' />
/// <reference path='../../../../third_party/typings/jasmine/jasmine.d.ts' />
var socks = require('../../socks-common/socks-headers');
var arraybuffers = require('../../../../third_party/uproxy-lib/arraybuffers/arraybuffers');
function slowTestDescription(useChurn) {
    var testerFactoryManager;
    var createTestModule = function (denyLocalhost) {
        return testerFactoryManager(denyLocalhost, useChurn);
    };
    beforeEach(function (done) {
        freedom('files/freedom-module.json', { 'debug': 'debug' }).then(function (freedomModuleFactoryManager) {
            testerFactoryManager = freedomModuleFactoryManager;
            done();
        });
    });
    afterEach(function () {
        expect(testerFactoryManager).not.toBeUndefined();
        // Close all created interfaces to the freedom module.
        testerFactoryManager.close();
    });
    // The default TCP SYN timeout is two minutes, so to be safe we
    // set a test timeout of four minutes.
    jasmine.DEFAULT_TIMEOUT_INTERVAL = 240000;
    // Opens 200 connections, sends 1 KB on each, and receives 250 KB on each
    it('download load test', function (done) {
        var blockSize = 1024;
        var testBlock = new ArrayBuffer(blockSize);
        var repeat = 250;
        var testModule = createTestModule();
        testModule.setRepeat(repeat);
        testModule.startEchoServer().then(function (port) {
            var connectionPromises = [];
            for (var i = 0; i < 200; ++i) {
                connectionPromises.push(testModule.connect(port));
            }
            return Promise.all(connectionPromises);
        }).then(function (connectionIds) {
            var completions = connectionIds.map(function (connectionId) {
                var resolve;
                var result = new Promise(function (F, R) {
                    resolve = F;
                });
                var isDone = false;
                var outputString = '';
                testModule.on('receivedData', function (event) {
                    if (event.connectionId != connectionId) {
                        return;
                    }
                    expect(isDone).toBe(false);
                    if (event.response.byteLength === repeat * blockSize) {
                        isDone = true;
                        resolve();
                    }
                });
                return testModule.sendData(connectionId, testBlock).then(function () {
                    return result;
                });
            });
            Promise.all(completions).then(done);
        });
    });
    // Opens 200 connections, sends 250 KB on each, and receives 0 KB on each
    it('upload load test', function (done) {
        var size = 250 * 1024;
        var testBlock = new ArrayBuffer(size);
        var testModule = createTestModule();
        testModule.setRepeat(0); // Don't send a reply at all.
        testModule.startEchoServer().then(function (port) {
            var connectionPromises = [];
            for (var i = 0; i < 200; ++i) {
                connectionPromises.push(testModule.connect(port));
            }
            return Promise.all(connectionPromises);
        }).then(function (connectionIds) {
            return connectionIds.map(function (connectionId) {
                return testModule.sendData(connectionId, testBlock);
            });
        }).then(function (sendResults) {
            return Promise.all(sendResults);
        }).catch(function (e) {
            expect(e).toBeUndefined();
        }).then(done);
    });
    it('100 MB echo load test', function (done) {
        var size = 100 * 1024 * 1024; // Larger than the 16 MB internal buffer in Chrome.
        var input = new ArrayBuffer(size);
        var testModule = createTestModule();
        testModule.startEchoServer().then(function (port) {
            return testModule.connect(port);
        }).then(function (connectionId) {
            return testModule.echo(connectionId, input);
        }).then(function (output) {
            expect(output.byteLength).toEqual(input.byteLength);
            expect(arraybuffers.byteEquality(input, output)).toBe(true);
        }).catch(function (e) {
            expect(e).toBeUndefined();
        }).then(done);
    });
    it('attempt to connect to a nonexistent IP address', function (done) {
        var testModule = createTestModule();
        // 192.0.2.0/24 is a reserved IP address range.
        testModule.connect(80, '192.0.2.111').then(function (connectionId) {
            // This code should not run, because this is a reserved IP address.
            expect(connectionId).toBeUndefined();
        }).catch(function (e) {
            // The socket should time out after two minutes.
            expect(e.reply).toEqual(6 /* TTL_EXPIRED */);
        }).then(done);
    });
}
describe('slow integration tests', function () {
    slowTestDescription(false);
});
describe('slow integration tests with churn', function () {
    slowTestDescription(true);
});
