/**
 * @fileoverview Debugging related routines.
 */


/**
 * Creates a browser data object with the given values.
 * @param {!string} type A string representing the "type" of this browser data
 *     object.
 * @param {!string} serverChallenge The server's challenge, as a base64-
 *     encoded string.
 * @param {!string} origin The server's origin, as seen by the browser.
 * @param {Object|string|undefined} opt_tlsChannelId TLS Channel Id
 * @return {string} A string representation of the browser data object.
 * @private
 */
function originStrippingMakeBrowserData_(
    type, serverChallenge, origin, opt_tlsChannelId) {
  var browserDataString = original_makeBrowserData(type, serverChallenge,
      origin, opt_tlsChannelId);
  var browserData = JSON.parse(browserDataString);
  delete browserData.origin;
  return JSON.stringify(browserData);
}

/**
 * Checks if a single appId can be asserted by the given origin.
 * @param {string} appId The appId to check
 * @return {Promise<boolean>} A promise for the result of the check
 * @private
 */
function allowAnyAppId_(appId) {
  return Promise.resolve(true);
}

var original_makeBrowserData;
var original_checkAppId;

function enableLocalTestingHacks() {
  window.HTTP_ORIGINS_ALLOWED = true;
  if (!original_makeBrowserData) {
    original_makeBrowserData = makeBrowserData;
    makeBrowserData = originStrippingMakeBrowserData_;
  }
  if (!original_checkAppId) {
    original_checkAppId = AppIdChecker.prototype.checkAppId_;
    AppIdChecker.prototype.checkAppId_ = allowAnyAppId_;
  }
}

function disableLocalTestingHacks() {
  window.HTTP_ORIGINS_ALLOWED = false;
  if (original_makeBrowserData) {
    makeBrowserData = original_makeBrowserData;
    original_makeBrowserData = undefined;
  }
  if (original_checkAppId) {
    AppIdChecker.prototype.checkAppId_ = original_checkAppId;
    original_checkAppId = undefined;
  }
}
