/// <reference path='../../../third_party/typings/es6-promise/es6-promise.d.ts' />
/// <reference path='../../../third_party/freedom-typings/udp-socket.d.ts' />
/**
 * Represents a UDP socket.
 * TODO: this is so similar to udprelay.ts that they can almost certainly
 *       be merged into one
 */
var UdpClient = (function () {
    function UdpClient(destAddress_, destPort_, onData_) {
        var _this = this;
        this.destAddress_ = destAddress_;
        this.destPort_ = destPort_;
        this.onData_ = onData_;
        this.onSocksClientData = function (recvFromInfo) {
            _this.onData_(recvFromInfo.data);
        };
        /**
         * Returns a promise to close the socket.
         */
        this.close = function () {
            return _this.socket.destroy();
        };
        /**
         * Returns the address on which the local socket associated with this
         * relay is listening.
         */
        this.getAddress = function () {
            return _this.address_;
        };
        /**
         * Returns the port on which the local socket associated with this
         * relay is listening.
         */
        this.getPort = function () {
            return _this.port_;
        };
        this.socket = freedom['core.udpsocket']();
    }
    /**
     * Returns a promise to create a socket, bind to the specified address and
     * port, and start relaying events. Specify port zero to have the system
     * choose a free port.
     */
    UdpClient.prototype.bind = function () {
        var _this = this;
        // TODO: not sure what else this should be?
        return this.socket.bind('127.0.0.1', 0).then(function (endpoint) {
            // Record the address and port on which our socket is listening.
            _this.address_ = endpoint.address;
            _this.port_ = endpoint.port;
            dbg('listening on : ' + JSON.stringify(endpoint));
            _this.socket.on('onData', _this.onSocksClientData);
            return endpoint;
        });
    };
    /**
     * Returns a promise to send data to the client.
     * This is intended for relaying responses from remote servers back to
     * the client.
     */
    UdpClient.prototype.send = function (buffer) {
        // TODO: throw error if socket not bound
        return this.socket.sendTo(buffer, this.destAddress_, this.destPort_);
    };
    return UdpClient;
})();
var modulePrefix_ = '[Net.UdpClient] ';
function dbg(msg) {
    console.log(modulePrefix_ + msg);
}
function dbgWarn(msg) {
    console.warn(modulePrefix_ + msg);
}
function dbgErr(msg) {
    console.error(modulePrefix_ + msg);
}
module.exports = UdpClient;
