/// <reference path='../../../third_party/typings/es6-promise/es6-promise.d.ts' />
// Sandwiches a call to an asynchronous function between two other
// (synchronous) functions. Intended for tracking the number of calls
// inflight to a freedom provider, which may fail to resolve if the
// provider's communication channel is destroyed while the call is
// pending (this can happen even on a normal call to the provider's
// close method).
// Throws synchronously if the before function raises an error; the
// returned promise rejects if the after function raises an error.
function wrap(wrapper, f) {
    try {
        wrapper.before();
    }
    catch (e) {
        wrapper.after();
        throw e;
    }
    return f().then(function (result) {
        wrapper.after();
        return result;
    }, function (e) {
        wrapper.after();
        throw e;
    });
}
exports.wrap = wrap;
var Wrapper = (function () {
    function Wrapper(destructor_) {
        var _this = this;
        this.destructor_ = destructor_;
        this.counter_ = 0;
        this.onceDestroyed_ = new Promise(function (F, R) {
            _this.fulfillDestroyed_ = F;
            _this.rejectDestroyed_ = R;
        });
        this.discard = function () {
            _this.after();
        };
        this.onceDestroyed = function () {
            return _this.onceDestroyed_;
        };
        this.before = function () {
            _this.counter_++;
        };
        this.after = function () {
            _this.counter_--;
            if (_this.counter_ < 0) {
                try {
                    _this.destructor_();
                    _this.fulfillDestroyed_();
                }
                catch (e) {
                    _this.rejectDestroyed_(e);
                }
            }
        };
    }
    return Wrapper;
})();
exports.Wrapper = Wrapper;
