/// <reference path='../../../../third_party/i18n/i18n.d.ts' />
// A little utility library for managing translation strings.
var i18nUtil;
(function (i18nUtil) {
    // Map of the supported languages to whether they are left-to-right or
    // right-to-left languages.
    // TODO: use enum instead of string for ltr/rtl.
    var languageDirection = {
        'en': 'ltr',
        'it': 'ltr',
        'ar': 'rtl',
        'fa': 'rtl'
    };
    // UI strings in the language selected by the user.
    var translatedStrings = {};
    // Clears the dictionary of UI strings (i.e. before a new language dictionary
    // is loaded).
    function clearTranslatedStrings() {
        for (var key in translatedStrings) {
            delete translatedStrings[key];
        }
    }
    // Given a Document, add translated strings to any text-containing child nodes.
    function translateStrings(node) {
        i18nTemplate.process(node, translatedStrings);
    }
    i18nUtil.translateStrings = translateStrings;
    // Retrieve messages.json file of the appropriate language and insert strings
    // into the application's UI.
    function changeLanguage(language) {
        clearTranslatedStrings();
        var xhr = new XMLHttpRequest();
        xhr.open('GET', 'locales/' + language + '/messages.json', true);
        xhr.onload = function () {
            if (this.readyState != 4) {
                return;
            }
            // Translate the JSON format to a simple { key : value, ... } dictionary.
            var retrievedMessages = JSON.parse(xhr.responseText);
            for (var key in retrievedMessages) {
                if (retrievedMessages.hasOwnProperty(key)) {
                    translatedStrings[key] = retrievedMessages[key].message;
                }
            }
            var htmlNode = document.querySelector('html');
            translateStrings(htmlNode);
            htmlNode.setAttribute('dir', languageDirection[language]);
        };
        xhr.send(null);
    }
    i18nUtil.changeLanguage = changeLanguage;
    // Return the language of the user's browser.
    //
    // TODO (lucyhe): find a better way to do this.
    function getBrowserLanguage() {
        return navigator.language.substring(0, 2);
    }
    i18nUtil.getBrowserLanguage = getBrowserLanguage;
})(i18nUtil || (i18nUtil = {}));
