/// <reference path='../../../../third_party/typings/es6-promise/es6-promise.d.ts' />
/// <reference path='../../../../third_party/freedom-typings/pgp.d.ts' />
/// <reference path='../../../../third_party/freedom-typings/freedom-common.d.ts' />
/// <reference path='../../../../third_party/freedom-typings/freedom-module-env.d.ts' />
var _this = this;
var arraybuffers = require('../../../../third_party/uproxy-lib/arraybuffers/arraybuffers');
var rtc_to_net = require('../../rtc-to-net/rtc-to-net');
var socks_to_rtc = require('../../socks-to-rtc/socks-to-rtc');
var logging = require('../../../../third_party/uproxy-lib/logging/logging');
var loggingTypes = require('../../../../third_party/uproxy-lib/loggingprovider/loggingprovider.types');
// Set each module to info, warn, error, or debug depending on which module
// you're debugging. Since the proxy outputs quite a lot of messages, show only
// warnings by default from the rest of the system.  Note that the proxy is
// extremely slow in debug mode.
freedom['loggingcontroller']().setDefaultFilter(0 /* console */, 1 /* info */);
freedom['loggingcontroller']().setFilters(0 /* console */, {
    'SocksToRtc': 1 /* info */,
    'RtcToNet': 1 /* info */
});
var log = new logging.Log('copypaste-socks');
var pgp = freedom['pgp']();
var friendKey;
// TODO interactive setup w/real passphrase
pgp.setup('', 'uProxy user <noreply@uproxy.org>');
var parentModule = freedom();
pgp.exportKey().then(function (publicKey) {
    parentModule.emit('publicKeyExport', publicKey);
});
var pcConfig = {
    iceServers: [{ urls: ['stun:stun.l.google.com:19302'] }, { urls: ['stun:stun1.l.google.com:19302'] }]
};
// These two modules together comprise a SOCKS server:
//  - socks-to-rtc is the frontend, which speaks the SOCKS protocol
//  - rtc-to-net creates sockets on behalf of socks-to-rtc
//
// The two modules communicate via a peer-to-peer connection.
//
// If we receive the 'start' signal from the UI then we create a
// socks-to-rtc module and this app will run the SOCKS frontend.
// If we receive signalling channel messages without having received
// the 'start' signal then we create an rtc-to-net instance and
// will act as the SOCKS backend.
var socksRtc;
var rtcNet;
parentModule.on('start', function () {
    var localhostEndpoint = { address: '127.0.0.1', port: 9999 };
    socksRtc = new socks_to_rtc.SocksToRtc();
    // Forward signalling channel messages to the UI.
    socksRtc.on('signalForPeer', function (signal) {
        parentModule.emit('signalForPeer', signal);
    });
    // SocksToRtc adds the number of bytes it sends/receives to its respective
    // queue as it proxies. When new numbers (of bytes) are added to these queues,
    // emit the number to the UI (look for corresponding freedom.on in main.html).
    socksRtc.on('bytesReceivedFromPeer', function (numBytes) {
        parentModule.emit('bytesReceived', numBytes);
    });
    socksRtc.on('bytesSentToPeer', function (numBytes) {
        parentModule.emit('bytesSent', numBytes);
    });
    socksRtc.on('stopped', function () {
        parentModule.emit('proxyingStopped');
    });
    socksRtc.startFromConfig(localhostEndpoint, pcConfig, false).then(function (endpoint) {
        log.info('socksRtc ready. listening to SOCKS5 on: ' + JSON.stringify(endpoint));
        log.info('` curl -x socks5h://localhost:9999 www.google.com `');
        parentModule.emit('proxyingStarted', endpoint);
    }).catch(function (e) {
        console.error('socksRtc Error: ' + e + '; ' + _this.socksRtc.toString());
    });
    log.info('created socks-to-rtc');
});
// Receive signalling channel messages from the UI.
// Messages are dispatched to either the socks-to-rtc or rtc-to-net
// modules depending on whether we're acting as the frontend or backend,
// respectively.
parentModule.on('handleSignalMessage', function (message) {
    if (socksRtc !== undefined) {
        socksRtc.handleSignalFromPeer(message);
    }
    else {
        if (rtcNet === undefined) {
            rtcNet = new rtc_to_net.RtcToNet();
            rtcNet.startFromConfig({ allowNonUnicast: true }, pcConfig, false); // obfuscate
            log.info('created rtc-to-net');
            // Forward signalling channel messages to the UI.
            rtcNet.signalsForPeer.setSyncHandler(function (message) {
                parentModule.emit('signalForPeer', message);
            });
            // Similarly to with SocksToRtc, emit the number of bytes sent/received
            // in RtcToNet to the UI.
            rtcNet.bytesReceivedFromPeer.setSyncHandler(function (numBytes) {
                parentModule.emit('bytesReceived', numBytes);
            });
            rtcNet.bytesSentToPeer.setSyncHandler(function (numBytes) {
                parentModule.emit('bytesSent', numBytes);
            });
            rtcNet.onceReady.then(function () {
                log.info('rtcNet ready.');
                parentModule.emit('proxyingStarted', null);
            });
            rtcNet.onceStopped.then(function () {
                parentModule.emit('proxyingStopped');
            });
        }
        rtcNet.handleSignalFromPeer(message);
    }
});
// Crypto request messages
parentModule.on('friendKey', function (newFriendKey) {
    friendKey = newFriendKey;
});
parentModule.on('signEncrypt', function (message) {
    pgp.signEncrypt(arraybuffers.stringToArrayBuffer(message), friendKey).then(function (cipherdata) {
        return pgp.armor(cipherdata);
    }).then(function (ciphertext) {
        parentModule.emit('ciphertext', ciphertext);
    });
});
parentModule.on('verifyDecrypt', function (ciphertext) {
    pgp.dearmor(ciphertext).then(function (cipherdata) {
        return pgp.verifyDecrypt(cipherdata, friendKey);
    }).then(function (result) {
        parentModule.emit('verifyDecryptResult', result);
    });
});
// Stops proxying.
parentModule.on('stop', function () {
    if (socksRtc !== undefined) {
        socksRtc.stop();
    }
    else if (rtcNet !== undefined) {
        rtcNet.stop();
    }
});
