/// <reference path='../../../../third_party/freedom-typings/freedom-common.d.ts' />
// A simple mock implementation of the freedom event handler that lets you cause
// fake events. Useful for mocking out freedom freedom modules that will raise
// events.
var MockFreedomEventHandler = (function () {
    function MockFreedomEventHandler(eventTypes_) {
        var _this = this;
        this.onHandlerTable_ = {};
        this.onceHandlerTable_ = {};
        eventTypes_.map(function (eventName) {
            _this.onHandlerTable_[eventName] = [];
            _this.onceHandlerTable_[eventName] = [];
        });
    }
    MockFreedomEventHandler.removeFromArray_ = function (x, a) {
        var index = a.indexOf(x);
        if (index > -1) {
            a.splice(index, 1);
        }
    };
    MockFreedomEventHandler.prototype.on = function (eventName, callback) {
        this.onHandlerTable_[eventName].push(callback);
    };
    MockFreedomEventHandler.prototype.once = function (eventName, callback) {
        this.onceHandlerTable_[eventName].push(callback);
    };
    // Off removes the event from both |onHandlerTable_| and |onceHandlerTable_|.
    // See: https://github.com/freedomjs/freedom/wiki/freedom.js-structure:-Consumer-Interface
    MockFreedomEventHandler.prototype.off = function (eventName, callback) {
        MockFreedomEventHandler.removeFromArray_(callback, this.onHandlerTable_[eventName]);
        MockFreedomEventHandler.removeFromArray_(callback, this.onceHandlerTable_[eventName]);
    };
    // Note: this is not part of the freedom interface.
    MockFreedomEventHandler.prototype.handleEvent = function (eventName, eventArgument) {
        this.onHandlerTable_[eventName].map(function (callback) {
            callback(eventArgument);
        });
        this.onceHandlerTable_[eventName].map(function (callback) {
            callback(eventArgument);
        });
        this.onceHandlerTable_[eventName] = [];
    };
    return MockFreedomEventHandler;
})();
module.exports = MockFreedomEventHandler;
