/// <reference path='../../../../third_party/freedom-typings/freedom-common.d.ts' />
var rtc_to_net = require('../../rtc-to-net/rtc-to-net');
var socks_to_rtc = require('../../socks-to-rtc/socks-to-rtc');
var tcp = require('../../net/tcp');
var socks = require('../../socks-common/socks-headers');
var arraybuffers = require('../../../../third_party/uproxy-lib/arraybuffers/arraybuffers');
// This abstract class is converted into a real class by Freedom, which
// fills in the unimplemented on(...) method in the process of
// constructing a module.
var AbstractProxyIntegrationTest = (function () {
    function AbstractProxyIntegrationTest(dispatchEvent_, denyLocalhost, obfuscate) {
        var _this = this;
        this.dispatchEvent_ = dispatchEvent_;
        this.echoServers_ = [];
        this.connections_ = {};
        this.localhost_ = '127.0.0.1';
        this.repeat_ = 1;
        this.startEchoServer = function () {
            var server = new tcp.Server({
                address: _this.localhost_,
                port: 0
            });
            server.connectionsQueue.setSyncHandler(function (tcpConnection) {
                tcpConnection.dataFromSocketQueue.setSyncHandler(function (buffer) {
                    var multiBuffer = [];
                    for (var i = 0; i < _this.repeat_; ++i) {
                        multiBuffer.push(buffer);
                    }
                    var concatenated = arraybuffers.concat(multiBuffer);
                    tcpConnection.send(concatenated);
                });
            });
            // Discard endpoint info; we'll get it again later via .onceListening().
            _this.echoServers_.push(server);
            return server.listen().then(function (endpoint) {
                return endpoint.port;
            });
        };
        this.setRepeat = function (repeat) {
            _this.repeat_ = repeat;
            return Promise.resolve();
        };
        this.startSocksPair_ = function (denyLocalhost, obfuscate) {
            var socksToRtcEndpoint = {
                address: _this.localhost_,
                port: 0
            };
            var rtcPcConfig = {
                iceServers: [],
            };
            var rtcToNetProxyConfig = {
                allowNonUnicast: !denyLocalhost // Allow RtcToNet to contact the localhost server.
            };
            _this.socksToRtc_ = new socks_to_rtc.SocksToRtc();
            _this.rtcToNet_ = new rtc_to_net.RtcToNet();
            _this.rtcToNet_.startFromConfig(rtcToNetProxyConfig, rtcPcConfig, obfuscate);
            _this.rtcToNet_.signalsForPeer.setSyncHandler(_this.socksToRtc_.handleSignalFromPeer);
            _this.socksToRtc_.on('signalForPeer', _this.rtcToNet_.handleSignalFromPeer);
            return _this.socksToRtc_.startFromConfig(socksToRtcEndpoint, rtcPcConfig, obfuscate);
        };
        // Assumes webEndpoint is IPv4.
        this.connectThroughSocks_ = function (socksEndpoint, webEndpoint) {
            var connection = new tcp.Connection({ endpoint: socksEndpoint });
            var authRequest = socks.composeAuthHandshakeBuffer([0 /* NOAUTH */]);
            connection.send(authRequest);
            var connected = new Promise(function (F, R) {
                connection.onceConnected.then(F);
                connection.onceClosed.then(R);
            });
            var firstBufferPromise = connection.receiveNext();
            return connected.then(function (i) {
                return firstBufferPromise;
            }).then(function (buffer) {
                var auth = socks.interpretAuthResponse(buffer);
                if (auth != 0 /* NOAUTH */) {
                    throw new Error('SOCKS server returned unexpected AUTH response.  ' + 'Expected NOAUTH (' + 0 /* NOAUTH */ + ') but got ' + auth);
                }
                var request = {
                    command: 1 /* TCP_CONNECT */,
                    endpoint: webEndpoint,
                };
                connection.send(socks.composeRequestBuffer(request));
                return connection.receiveNext();
            }).then(function (buffer) {
                var response = socks.interpretResponseBuffer(buffer);
                if (response.reply != 0 /* SUCCEEDED */) {
                    // TODO: Fix bad style: reject should only and always be an error.
                    // We should be resolving with result status.
                    return Promise.reject(response);
                }
                return Promise.resolve(connection);
            });
        };
        this.connect = function (port, address) {
            try {
                return _this.socksEndpoint_.then(function (socksEndpoint) {
                    var echoEndpoint = {
                        address: address || _this.localhost_,
                        port: port
                    };
                    return _this.connectThroughSocks_(socksEndpoint, echoEndpoint);
                }).then(function (connection) {
                    _this.connections_[connection.connectionId] = connection;
                    return connection.connectionId;
                });
            }
            catch (e) {
                return Promise.reject(e);
            }
        };
        this.echo = function (connectionId, content) {
            return _this.echoMultiple(connectionId, [content]).then(function (responses) {
                return arraybuffers.concat(responses);
            });
        };
        this.echoMultiple = function (connectionId, contents) {
            try {
                var connection = _this.connections_[connectionId];
                contents.forEach(connection.send);
                var received = [];
                var bytesReceived = 0;
                var bytesToSend = 0;
                contents.forEach(function (content) {
                    bytesToSend += content.byteLength;
                });
                return new Promise(function (F, R) {
                    connection.dataFromSocketQueue.setSyncHandler(function (echo) {
                        received.push(echo);
                        bytesReceived += echo.byteLength;
                        if (bytesReceived == bytesToSend) {
                            F(received);
                        }
                    });
                });
            }
            catch (e) {
                return Promise.reject(e);
            }
        };
        this.sendData = function (connectionId, content) {
            try {
                var connection = _this.connections_[connectionId];
                connection.send(content);
                connection.dataFromSocketQueue.setSyncHandler(function (response) {
                    _this.dispatchEvent_('receivedData', {
                        connectionId: connectionId,
                        response: response
                    });
                });
                return Promise.resolve();
            }
            catch (e) {
                return Promise.reject(e);
            }
        };
        this.on = function (name, listener) {
            throw new Error('Placeholder function to keep Typescript happy');
        };
        this.socksEndpoint_ = this.startSocksPair_(denyLocalhost, obfuscate);
    }
    return AbstractProxyIntegrationTest;
})();
module.exports = AbstractProxyIntegrationTest;
