/// <reference path='../../../third_party/freedom-typings/freedom-common.d.ts' />
/// <reference path='../../../third_party/typings/es6-promise/es6-promise.d.ts' />
/// <reference path='../../../third_party/freedom-typings/udp-socket.d.ts' />
// TODO(ldixon): reorganize the utransformers and rename uproxy-obfuscators.
// Ideal:
//  import Transformer = require('uproxy-obfuscators/transformer');
//  import Rabbit = require('uproxy-obfuscators/rabbit.transformer');
//  import Fte = require('uproxy-obfuscators/fte.transformer');
// Current:
/// <reference path='../../../third_party/uTransformers/utransformers.d.ts' />
// import Rabbit = require('utransformers/src/transformers/uTransformers.fte');
// import Fte = require('utransformers/src/transformers/uTransformers.rabbit');
var PassThrough = require('../simple-transformers/passthrough');
var CaesarCipher = require('../simple-transformers/caesar');
var logging = require('../../../third_party/uproxy-lib/logging/logging');
var log = new logging.Log('churn-pipe');
/**
 * Listens on a port for UDP datagrams -- emitting a Freedom message for each
 * datagram received -- and sends UDP datagrams to a destination, in response
 * to Freedom messages.
 *
 * Each incoming and outgoing message is first passed through an obfuscator.
 * The obfuscator is used via direct function calls rather than Freedom
 * message passing owing to the inelegance of receiving a response *back*
 * from a Freedom module.
 */
var Pipe = (function () {
    // TODO: define a type for event dispatcher in freedom-typescript-api
    function Pipe(dispatchEvent_) {
        var _this = this;
        this.dispatchEvent_ = dispatchEvent_;
        /**
         * Returns a promise to create a socket, bind to the specified address, and
         * start listening for datagrams.
         */
        this.bind = function (localAddress, localPort, remoteAddress, remotePort, transformerName, key, config) {
            try {
                _this.transformer_ = _this.makeTransformer_(transformerName, key, config);
            }
            catch (e) {
                return Promise.reject(e);
            }
            // Next, bind to a socket.
            _this.remoteEndpoint_ = {
                address: remoteAddress,
                port: remotePort
            };
            return _this.socket_.bind(localAddress, localPort).then(function (resultCode) {
                if (resultCode != 0) {
                    return Promise.reject(new Error('listen failed with result code ' + resultCode));
                }
                _this.socket_.on('onData', _this.onData_);
            });
        };
        this.makeTransformer_ = function (
            // Name of transformer to use, e.g. 'rabbit' or 'none'.
            name, 
            // Key for transformer, if any.
            key, 
            // JSON-encoded configuration, if any.
            config) {
            var transformer;
            // TODO(ldixon): re-enable rabbit and FTE once we can figure out why they
            // don't load in freedom.
            /* if (name == 'rabbit') {
              transformer = Rabbit.Transformer();
            } else if (name == 'fte') {
              transformer = Fte.Transformer();
            } else */ if (name == 'caesar') {
                transformer = new CaesarCipher();
            }
            else if (name == 'none') {
                transformer = new PassThrough();
            }
            else {
                throw new Error('unknown transformer: ' + name);
            }
            if (key) {
                transformer.setKey(key);
            }
            if (config) {
                transformer.configure(config);
            }
            return transformer;
        };
        /**
         * Sends a message over the network to the remote side.
         * The message is obfuscated before it hits the wire.
         */
        this.send = function (buffer) {
            return _this.sendTo(buffer, _this.remoteEndpoint_);
        };
        /**
         * Sends a message over the network to the specified destination.
         * The message is obfuscated before it hits the wire.
         */
        this.sendTo = function (buffer, to) {
            var transformedBuffer = _this.transformer_.transform(buffer);
            return _this.socket_.sendTo(transformedBuffer, to.address, to.port).then(function () {
                return Promise.resolve();
            });
        };
        this.getLocalEndpoint = function () {
            return _this.socket_.getInfo().then(function (socketInfo) {
                return {
                    address: socketInfo.localAddress,
                    port: socketInfo.localPort
                };
            });
        };
        /**
         * Called when a message is received over the network from the remote side.
         * The message is de-obfuscated before the Freedom message is emitted.
         */
        this.onData_ = function (recvFromInfo) {
            var transformedBuffer = recvFromInfo.data;
            var buffer = _this.transformer_.restore(transformedBuffer);
            _this.dispatchEvent_('message', {
                data: buffer,
                source: {
                    address: recvFromInfo.address,
                    port: recvFromInfo.port
                }
            });
        };
        // TODO: clean up udp sockets
        this.socket_ = freedom['core.udpsocket']();
    }
    return Pipe;
})();
module.exports = Pipe;
