/// <reference path='../../../../third_party/polymer/polymer.d.ts' />
/// <reference path='../../../../third_party/typings/es6-promise/es6-promise.d.ts' />
/// <reference path='../../../../third_party/freedom-typings/pgp.d.ts' />
/// <reference path='../../../../third_party/freedom-typings/freedom-common.d.ts' />
/// <reference path='../../../../third_party/freedom-typings/freedom-core-env.d.ts' />
var arraybuffers = require('../../../../third_party/uproxy-lib/arraybuffers/arraybuffers');
var copypaste_module;
(function (copypaste_module) {
    copypaste_module.onceReady = freedom('freedom-module.json', {
        'logger': 'uproxy-lib/loggingprovider/freedom-module.json',
        'debug': 'debug'
    }).then(function (copypasteSocksFactory) {
        var copypaste = copypasteSocksFactory();
        copypaste.on('signalForPeer', function (message) {
            copypaste_module.model.readyForStep2 = true;
            // Append the new signalling message to the previous message(s), if any.
            // Base64-encode the concatenated messages because some communication
            // channels are likely to transform portions of the raw concatenated JSON
            // into emoticons, whereas the base64 alphabet is much less prone to such
            // unintended transformation.
            var oldConcatenatedJson = base64Decode(copypaste_module.model.outboundMessageValue.trim());
            var newConcatenatedJson = oldConcatenatedJson + '\n' + JSON.stringify(message);
            if (copypaste_module.model.usingCrypto) {
                copypaste.emit('friendKey', copypaste_module.model.friendPublicKey);
                copypaste.emit('signEncrypt', base64Encode(newConcatenatedJson));
            }
            copypaste_module.model.outboundMessageValue = base64Encode(newConcatenatedJson);
        });
        copypaste.on('publicKeyExport', function (publicKey) {
            copypaste_module.model.userPublicKey = publicKey;
        });
        copypaste.on('ciphertext', function (ciphertext) {
            copypaste_module.model.outboundMessageValue = ciphertext;
        });
        copypaste.on('verifyDecryptResult', function (result) {
            copypaste_module.model.inputDecrypted = true;
            copypaste_module.model.inputSigned = result.signedBy[0] == copypaste_module.model.friendUserId;
            copypaste_module.model.inboundText = arraybuffers.arrayBufferToString(result.data);
            parseInboundMessages(copypaste_module.model.inboundText);
        });
        copypaste.on('bytesReceived', function (numNewBytesReceived) {
            copypaste_module.model.totalBytesReceived += numNewBytesReceived;
        });
        copypaste.on('bytesSent', function (numNewBytesSent) {
            copypaste_module.model.totalBytesSent += numNewBytesSent;
        });
        copypaste.on('proxyingStarted', function (listeningEndpoint) {
            if (listeningEndpoint !== null) {
                copypaste_module.model.endpoint = listeningEndpoint.address + ':' + listeningEndpoint.port;
            }
            copypaste_module.model.proxyingState = 'started';
        });
        copypaste.on('proxyingStopped', function () {
            copypaste_module.model.proxyingState = 'stopped';
        });
        return copypaste;
    }, function (e) {
        console.error('could not load freedom: ' + e.message);
    });
    // TODO: use actual interaction to define user-ids.
    copypaste_module.model = {
        givingOrGetting: null,
        usingCrypto: false,
        inputDecrypted: false,
        inputSigned: false,
        userPublicKey: '',
        friendPublicKey: '',
        friendUserId: 'Joe <joe@test.com>',
        readyForStep2: false,
        outboundMessageValue: '',
        inboundText: '',
        inputIsWellFormed: false,
        proxyingState: 'notYetAttempted',
        endpoint: null,
        totalBytesReceived: 0,
        totalBytesSent: 0
    };
    // Stores the parsed messages for use later, if & when the user clicks the
    // button for consuming the messages.
    var parsedInboundMessages;
    // Define basee64 helper functions that are type-annotated and meaningfully
    // named.
    function base64Encode(unencoded) {
        return window.btoa(unencoded);
    }
    // Throws an exception if the input is malformed.
    function base64Decode(encoded) {
        return window.atob(encoded);
    }
    // Parses the contents of the form field 'inboundMessageField' as a sequence of
    // signalling messages. Enables/disables the corresponding form button, as
    // appropriate. Returns null if the field contents are malformed.
    function parseInboundMessages(inboundMessageFieldValue) {
        // Base64-decode the pasted text.
        var signalsString = null;
        try {
            signalsString = base64Decode(inboundMessageFieldValue.trim());
        }
        catch (e) {
            // TODO: Notify the user that the pasted text is malformed.
            return null;
        }
        var signals = signalsString.trim().split('\n');
        // Each line should be a JSON representation of a signals.Message.
        // Parse the lines here.
        var parsedSignals = [];
        for (var i = 0; i < signals.length; i++) {
            var s = signals[i].trim();
            // TODO: Consider detecting the error if the text is well-formed JSON but
            // does not represent a signals.Message.
            var message;
            try {
                message = JSON.parse(s);
            }
            catch (e) {
                parsedSignals = null;
                break;
            }
            parsedSignals.push(message);
        }
        // Enable/disable, as appropriate, the button for consuming the messages.
        if (null !== parsedSignals && parsedSignals.length > 0) {
            copypaste_module.model.inputIsWellFormed = true;
        }
        else {
        }
        parsedInboundMessages = parsedSignals;
    }
    copypaste_module.parseInboundMessages = parseInboundMessages;
    // Forwards each line from the paste box to the Freedom app, which
    // interprets each as a signalling channel message. The Freedom app
    // knows whether this message should be sent to the socks-to-rtc
    // or rtc-to-net module. Disables the form field.
    function consumeInboundMessage() {
        // Forward the signalling messages to the Freedom app.
        copypaste_module.onceReady.then(function (copypaste) {
            for (var i = 0; i < parsedInboundMessages.length; i++) {
                copypaste.emit('handleSignalMessage', parsedInboundMessages[i]);
            }
        });
        copypaste_module.model.proxyingState = 'connecting';
        // TODO: Report success/failure to the user.
    }
    copypaste_module.consumeInboundMessage = consumeInboundMessage;
    ;
    function verifyDecryptInboundMessage(ciphertext) {
        copypaste_module.onceReady.then(function (copypaste) {
            copypaste.emit('friendKey', copypaste_module.model.friendPublicKey);
            copypaste.emit('verifyDecrypt', ciphertext);
        });
    }
    copypaste_module.verifyDecryptInboundMessage = verifyDecryptInboundMessage;
    ;
})(copypaste_module || (copypaste_module = {})); // module copypaste_api
exports.copypaste = copypaste_module;
