// For testing just the TCP server portion (see src/client/tcp.ts)
var arraybuffers = require('../../../third_party/uproxy-lib/arraybuffers/arraybuffers');
var tcp = require('../net/tcp');
var logging = require('../../../third_party/uproxy-lib/logging/logging');
var log = new logging.Log('echo-server');
var TcpEchoServer = (function () {
    function TcpEchoServer(endpoint) {
        var _this = this;
        this.endpoint = endpoint;
        this.onConnection_ = function (conn) {
            log.info(conn.toString() + ': New TCP Connection: ');
            // The onceConnected is fulfilled by onConnection (in practice, but not
            // specified by the freedom TCP interface)
            conn.onceConnected.then(function (endpoint) {
                log.info(conn.toString() + ': Connection resolved to: ' + JSON.stringify(endpoint));
            });
            // This use of |receiveNext| here is to shows you can how to use it to get
            // the first ArrayBuffer of data and treat handling it differently.
            conn.receiveNext().then(function (data) {
                log.info(conn.toString() + ': Received first data!');
                _this.onData_(conn, data);
                // Now handle further data as we get it using |this.onData_|.
                conn.dataFromSocketQueue.setSyncHandler(_this.onData_.bind(_this, conn));
            });
        };
        this.onData_ = function (conn, data) {
            log.info(conn.toString() + ': Received: ' + data.byteLength + " bytes.");
            var hexStrOfData = arraybuffers.arrayBufferToHexString(data);
            log.info(conn.toString() + ': Received data as hex-string: ' + hexStrOfData);
            // This shows how you handle some data and close the connection.
            if (hexStrOfData === TcpEchoServer.CTRL_D_HEX_STR_CODE) {
                conn.close();
                return;
            }
            conn.send(data);
        };
        this.server = new tcp.Server(endpoint);
        // Start listening to connections.
        this.server.listen().then(function (listeningEndpoint) {
            log.info('listening on %1', listeningEndpoint);
            _this.server.onceShutdown().then(function (kind) {
                log.info('server shutdown: %1', tcp.SocketCloseKind[kind]);
            });
        }).catch(function (e) {
            log.error('failed to listen on %1: %2', endpoint, e.toString);
        });
        // Handle any new connections using |this.onConnection_|.
        this.server.connectionsQueue.setSyncHandler(this.onConnection_);
    }
    // '4' is the char-code for control-D which we use to close the TCP
    // connection.
    TcpEchoServer.CTRL_D_HEX_STR_CODE = '4';
    return TcpEchoServer;
})();
module.exports = TcpEchoServer;
