/// <reference path='../../../third_party/typings/es6-promise/es6-promise.d.ts' />
/// <reference path='../../../third_party/freedom-typings/freedom-module-env.d.ts' />
var rtc_to_net = require('../rtc-to-net/rtc-to-net');
var socks_to_rtc = require('../socks-to-rtc/socks-to-rtc');
var logging = require('../../../third_party/uproxy-lib/logging/logging');
var loggingTypes = require('../../../third_party/uproxy-lib/loggingprovider/loggingprovider.types');
exports.moduleName = 'simple-socks';
exports.log = new logging.Log(exports.moduleName);
// Set each module to info, warn, error, or debug depending on which module
// you're debugging. Since the proxy outputs quite a lot of messages, show only
// warnings by default from the rest of the system.  Note that the proxy is
// extremely slow in debug mode.
exports.loggingController = freedom['loggingcontroller']();
// Example to show how to manuall configure console filtering.
exports.loggingController.setDefaultFilter(0 /* console */, 1 /* info */);
exports.loggingController.setFilters(0 /* console */, {
    'simple-socks': 0 /* debug */,
    'SocksToRtc': 1 /* info */,
    'RtcToNet': 1 /* info */
});
//-----------------------------------------------------------------------------
var localhostEndpoint = { address: '127.0.0.1', port: 9999 };
//-----------------------------------------------------------------------------
var pcConfig = {
    iceServers: [{ urls: ['stun:stun.l.google.com:19302'] }, { urls: ['stun:stun1.l.google.com:19302'] }]
};
exports.rtcNet = new rtc_to_net.RtcToNet();
exports.rtcNet.startFromConfig({ allowNonUnicast: true }, pcConfig, false); // obfuscate
//-----------------------------------------------------------------------------
exports.socksRtc = new socks_to_rtc.SocksToRtc();
exports.socksRtc.on('signalForPeer', exports.rtcNet.handleSignalFromPeer);
exports.socksRtc.startFromConfig(localhostEndpoint, pcConfig, false).then(function (endpoint) {
    exports.log.info('SocksToRtc listening on: ' + JSON.stringify(endpoint));
    exports.log.info('curl -x socks5h://' + endpoint.address + ':' + endpoint.port + ' www.example.com');
}, function (e) {
    exports.log.error('failed to start SocksToRtc: ' + e.message);
});
//-----------------------------------------------------------------------------
var getterBytesReceived = 0;
var getterBytesSent = 0;
var giverBytesReceived = 0;
var giverBytesSent = 0;
exports.rtcNet.signalsForPeer.setSyncHandler(exports.socksRtc.handleSignalFromPeer);
// TODO: Re-enable received/sent messages when per-component logging is fixed:
//         https://github.com/uProxy/uproxy/issues/906
exports.socksRtc.on('bytesReceivedFromPeer', function (numBytes) {
    getterBytesReceived += numBytes;
    // log.debug('Getter received ' + numBytes + ' bytes. (Total received: '
    //   + getterBytesReceived + ' bytes)');
});
exports.socksRtc.on('bytesSentToPeer', function (numBytes) {
    getterBytesSent += numBytes;
    // log.debug('Getter sent ' + numBytes + ' bytes. (Total sent: '
    //   + getterBytesSent + ' bytes)');
});
exports.rtcNet.bytesReceivedFromPeer.setSyncHandler(function (numBytes) {
    giverBytesReceived += numBytes;
    // log.debug('Giver received ' + numBytes + ' bytes. (Total received: '
    //   + giverBytesReceived + ' bytes)');
});
exports.rtcNet.bytesSentToPeer.setSyncHandler(function (numBytes) {
    giverBytesSent += numBytes;
    // log.debug('Giver sent ' + numBytes + ' bytes. (Total sent: '
    //   + giverBytesSent + ' bytes)');
});
exports.rtcNet.onceReady.then(function () {
    exports.log.info('RtcToNet ready');
}, function (e) {
    exports.log.error('failed to start RtcToNet: ' + e.message);
});
