/// <reference path='../../../third_party/typings/es6-promise/es6-promise.d.ts' />
/// <reference path='../../../third_party/typings/jasmine/jasmine.d.ts' />
var TurnFrontend = require('./turn-frontend');
var messages = require('./messages');
describe("turn frontend", function () {
    // Returns an array of 12 bytes, suuitable for use as a STUN/TURN
    // transaction ID.
    function getTransactionIdBytes() {
        return new Uint8Array([
            0x2f,
            0x68,
            0x65,
            0x79,
            0x6b,
            0x6b,
            0x31,
            0x54,
            0x46,
            0x32,
            0x36,
            0x57
        ]);
    }
    var frontend;
    var endpoint;
    beforeEach(function () {
        frontend = new TurnFrontend();
        endpoint = {
            address: '127.0.0.1',
            port: 10000
        };
    });
    // Unsupported requests should reject.
    it('reject unsupported request', function (done) {
        var request = {
            method: 999,
            clazz: 1 /* REQUEST */,
            transactionId: getTransactionIdBytes(),
            attributes: []
        };
        frontend.handleStunMessage(request, endpoint).catch(done);
    });
    // Treat any ALLOCATE requests without a USERNAME attribute
    // as the "initial ALLOCATE request" which should return a
    // failure, with NONCE and REALM attributes.
    it('initial allocate request', function (done) {
        var request = {
            method: 3 /* ALLOCATE */,
            clazz: 1 /* REQUEST */,
            transactionId: getTransactionIdBytes(),
            attributes: [{
                type: 25 /* REQUESTED_TRANSPORT */
            }]
        };
        frontend.handleStunMessage(request, endpoint).then(function (response) {
            expect(response.method).toEqual(3 /* ALLOCATE */);
            expect(response.clazz).toEqual(3 /* FAILURE_RESPONSE */);
            // TODO: inspect these attributes
            messages.findFirstAttributeWithType(9 /* ERROR_CODE */, response.attributes);
            messages.findFirstAttributeWithType(21 /* NONCE */, response.attributes);
            messages.findFirstAttributeWithType(20 /* REALM */, response.attributes);
        }).then(done);
    });
    // TODO: test second allocate request (requires socket mocks)
    // TODO: test repeat ALLOCATE requests, verify just one allocation
    // TODO: test socket creation failure
    // TODO: test create permission returns success
});
