/// <reference path='../../../../third_party/freedom-typings/freedom-module-env.d.ts' />
var arraybuffers = require('../../../../third_party/uproxy-lib/arraybuffers/arraybuffers');
var tcp = require('../../net/tcp');
var logging = require('../../../../third_party/uproxy-lib/logging/logging');
var loggingTypes = require('../../../../third_party/uproxy-lib/loggingprovider/loggingprovider.types');
exports.loggingController = freedom['loggingcontroller']();
exports.loggingController.setDefaultFilter(0 /* console */, 0 /* debug */);
exports.moduleName = 'integration-tests/tcp';
exports.log = new logging.Log(exports.moduleName);
var getServerOnFreePort = function () {
    return new tcp.Server({
        address: '127.0.0.1',
        port: 0
    });
};
exports.parentModule = freedom();
// Starts an echo server on a free port and sends some data to the server,
// verifying that an echo is received.
exports.parentModule.on('listen', function () {
    var server = getServerOnFreePort();
    server.connectionsQueue.setSyncHandler(function (tcpConnection) {
        exports.log.info('New TCP connection: ' + tcpConnection.toString());
        tcpConnection.dataFromSocketQueue.setSyncHandler(function (buffer) {
            tcpConnection.send(buffer);
        });
    });
    server.listen().then(function (endpoint) {
        var client = new tcp.Connection({ endpoint: endpoint });
        client.dataFromSocketQueue.setSyncNextHandler(function (buffer) {
            var s = arraybuffers.arrayBufferToString(buffer);
            if (s == 'ping') {
                exports.parentModule.emit('listen');
            }
        });
        client.onceConnected.then(function (info) {
            client.send(arraybuffers.stringToArrayBuffer('ping'));
        });
    });
});
// Starts a server on a free port and makes a connection to that
// port before shutting down the server, verifying that onceShutdown
// fulfills.
exports.parentModule.on('shutdown', function () {
    var server = getServerOnFreePort();
    server.listen().then(function (endpoint) {
        var client = new tcp.Connection({ endpoint: endpoint });
        server.connectionsQueue.setSyncHandler(function (connection) {
            client.onceConnected.then(function () {
                server.shutdown();
                return Promise.all([connection.onceClosed, client.onceClosed, server.onceShutdown()]);
            }).then(function (values) {
                exports.parentModule.emit('shutdown');
            });
        });
    });
});
// Starts a server on a free port and makes a connection to that
// port before closing that connection, verifying that each side
// of the socket receives the appropriate SocketCloseKind event.
exports.parentModule.on('onceclosedbyserver', function () {
    var server = getServerOnFreePort();
    server.listen().then(function (endpoint) {
        var client = new tcp.Connection({ endpoint: endpoint });
        server.connectionsQueue.setSyncHandler(function (connection) {
            client.onceConnected.then(function () {
                connection.close();
                return Promise.all([connection.onceClosed, client.onceClosed]);
            }).then(function (values) {
                if (values[0] === 0 /* WE_CLOSED_IT */ && values[1] === 1 /* REMOTELY_CLOSED */) {
                    exports.parentModule.emit('onceclosedbyserver');
                }
            });
        });
    });
});
// Starts a server on a free port and makes a connection to that
// port before closing that connection, verifying that each side
// of the socket receives the appropriate SocketCloseKind event.
exports.parentModule.on('onceclosedbyclient', function () {
    var server = getServerOnFreePort();
    server.listen().then(function (endpoint) {
        var client = new tcp.Connection({ endpoint: endpoint });
        server.connectionsQueue.setSyncHandler(function (connection) {
            client.onceConnected.then(function () {
                client.close();
                return Promise.all([connection.onceClosed, client.onceClosed]);
            }).then(function (values) {
                if (values[0] === 1 /* REMOTELY_CLOSED */ && values[1] === 0 /* WE_CLOSED_IT */) {
                    exports.parentModule.emit('onceclosedbyclient');
                }
            });
        });
    });
});
// Attempts to connect to an address which is not bound.
exports.parentModule.on('neverconnected', function () {
    var client = new tcp.Connection({
        endpoint: {
            address: '127.0.0.1',
            port: 1023 // Reserved port.
        }
    });
    client.onceConnected.catch(function (e) {
        return client.onceClosed;
    }).then(function (kind) {
        if (kind === 2 /* NEVER_CONNECTED */) {
            exports.parentModule.emit('neverconnected');
        }
    });
});
// Starts an echo server on a free port and verifies that five echo clients
// can send and receive data from the server.
exports.parentModule.on('multipleclients', function () {
    var server = getServerOnFreePort();
    server.connectionsQueue.setSyncHandler(function (tcpConnection) {
        tcpConnection.dataFromSocketQueue.setSyncHandler(function (buffer) {
            tcpConnection.send(buffer);
        });
    });
    server.listen().then(function (endpoint) {
        var addEchoClient = function (i) {
            var fulfill;
            var client = new tcp.Connection({ endpoint: endpoint });
            client.dataFromSocketQueue.setSyncNextHandler(function (buffer) {
                var bytes = new Uint8Array(buffer);
                if (bytes.length == 1 && bytes[0] == i) {
                    fulfill();
                }
            });
            client.onceConnected.then(function (info) {
                var bytes = new Uint8Array([i]);
                client.send(bytes.buffer);
            });
            return new Promise(function (F, R) {
                fulfill = F;
            });
        };
        var promises = [];
        for (var i = 0; i < 5; i++) {
            promises.push(addEchoClient(i));
        }
        Promise.all(promises).then(function (answers) {
            exports.parentModule.emit('multipleclients');
        });
    });
});
// Starts an echo server on a free port and verifies that its connectionsCount
// is correct once five clients have connected to it.
exports.parentModule.on('connectionscount', function () {
    var server = getServerOnFreePort();
    server.listen().then(function (endpoint) {
        var clients = [];
        for (var i = 0; i < 5; i++) {
            clients.push(new tcp.Connection({ endpoint: endpoint }));
        }
        Promise.all(clients.map(function (client) {
            return client.onceConnected;
        })).then(function (answers) {
            if (server.connectionsCount() != clients.length) {
                throw new Error();
            }
        }).then(function () {
            exports.parentModule.emit('connectionscount');
        });
    });
});
