/// <reference path='../../../third_party/typings/es6-promise/es6-promise.d.ts' />
/// <reference path='../../../third_party/freedom-typings/freedom-common.d.ts' />
/// <reference path='../../../third_party/freedom-typings/freedom-module-env.d.ts' />
var peerconnection = require('../../../third_party/uproxy-lib/webrtc/peerconnection');
var handler = require('../../../third_party/uproxy-lib/handler/queue');
var churn = require('../churn/churn');
var tcp = require('../net/tcp');
var socks = require('../socks-common/socks-headers');
var Pool = require('../pool/pool');
var logging = require('../../../third_party/uproxy-lib/logging/logging');
// SocksToRtc passes socks requests over WebRTC datachannels.
var SocksToRtc;
(function (_SocksToRtc) {
    var log = new logging.Log('SocksToRtc');
    // The |SocksToRtc| class runs a SOCKS5 proxy server which passes requests
    // remotely through WebRTC peer connections.
    // TODO: rename this 'Server'.
    // TODO: Extract common code for this and SocksToRtc:
    //         https://github.com/uProxy/uproxy/issues/977
    var SocksToRtc = (function () {
        // CONSIDER: Remove |on| and |listeners_| once all users of this class use
        // it via freedom, or determine a better long-term plan for supporting
        // events compatibly with and without freedom
        // (https://github.com/uProxy/uproxy/issues/733).
        // Note: The optional |dispatchEvent_| is for when this class is loaded as a
        // freedom module.
        function SocksToRtc(dispatchEvent_) {
            var _this = this;
            this.dispatchEvent_ = dispatchEvent_;
            this.onceStopping_ = new Promise(function (F, R) {
                _this.fulfillStopping_ = F;
            });
            // The two Queues below only count bytes transferred between the SOCKS
            // client and the remote host(s) the client wants to connect to. WebRTC
            // overhead (DTLS headers, ICE initiation, etc.) is not included (because
            // WebRTC does not provide easy access to that data) nor is SOCKS
            // protocol-related data (because it's sent via string messages).
            // All Sessions created in one instance of SocksToRtc will share and
            // push numbers to the same queues (belonging to that instance of SocksToRtc).
            // Queue of the number of bytes received from the peer. Handler is typically
            // defined in the class that creates an instance of SocksToRtc.
            this.bytesReceivedFromPeer_ = new handler.Queue();
            // Queue of the number of bytes sent to the peer. Handler is typically
            // defined in the class that creates an instance of SocktsToRtc.
            this.bytesSentToPeer_ = new handler.Queue();
            // Handles creation of a TCP server and peerconnection. Returns the endpoint
            // it ended up listening on (if |localSocksServerEndpoint| has port set to
            // 0, then a dynamic port is allocated and this port is returned within the
            // promise's endpoint).  NOTE: Users of this class MUST add on-event
            // listeners before calling this method.
            this.startFromConfig = function (localSocksServerEndpoint, pcConfig, obfuscate) {
                var pc = freedom['core.rtcpeerconnection'](pcConfig);
                return _this.start(new tcp.Server(localSocksServerEndpoint), obfuscate ? new churn.Connection(pc, 'SocksToRtc') : new peerconnection.PeerConnectionClass(pc));
            };
            // Starts the SOCKS server with the supplied TCP server and peerconnection.
            // Returns a promise that resolves when the server is ready to use. This
            // method is public only for testing purposes.
            this.start = function (tcpServer, 
                // TODO(iislucas): are the types correct here? Does an obfuscated
                // channel have a different signalling type?
                peerconnection) {
                if (_this.tcpServer_) {
                    throw new Error('already configured');
                }
                _this.tcpServer_ = tcpServer;
                _this.tcpServer_.connectionsQueue.setSyncHandler(_this.makeTcpToRtcSession_);
                _this.peerConnection_ = peerconnection;
                _this.pool_ = new Pool(_this.peerConnection_, 'SocksToRtc');
                _this.peerConnection_.signalForPeerQueue.setSyncHandler(_this.dispatchEvent_.bind(_this, 'signalForPeer'));
                _this.bytesSentToPeer_.setSyncHandler(_this.dispatchEvent_.bind(_this, 'bytesSentToPeer'));
                _this.bytesReceivedFromPeer_.setSyncHandler(_this.dispatchEvent_.bind(_this, 'bytesReceivedFromPeer'));
                // Start and listen for notifications.
                peerconnection.negotiateConnection();
                var onceReady = Promise.all([
                    tcpServer.listen(),
                    peerconnection.onceConnected
                ]).then(function (answers) {
                    return tcpServer.onceListening();
                });
                // Shutdown if startup fails or when the server socket or
                // peerconnection terminates.
                onceReady.catch(_this.fulfillStopping_);
                _this.tcpServer_.onceShutdown().then(function (kind) {
                    log.info('server socket closed: %1', tcp.SocketCloseKind[kind]);
                }).then(_this.fulfillStopping_);
                _this.peerConnection_.onceDisconnected.then(function () {
                    log.info('peerconnection terminated');
                }, function (e) {
                    log.error('peerconnection terminated with error: %1', [e.message]);
                }).then(_this.fulfillStopping_);
                _this.onceStopped_ = _this.onceStopping_.then(_this.stopResources_);
                _this.onceStopped_.then(_this.dispatchEvent_.bind(_this, 'stopped'));
                var rejectOnStopping = new Promise(function (F, R) {
                    _this.onceStopping_.then(R);
                });
                return Promise.race([onceReady, rejectOnStopping]);
            };
            // Initiates shutdown of the TCP server and peerconnection.
            // Returns onceStopped.
            this.stop = function () {
                log.info('stop requested');
                _this.fulfillStopping_();
                return _this.onceStopped_;
            };
            // An implementation of dispatchEvent to use if none has been provided
            // (i.e. when this class is not being used as a freedom    // module).
            // For simplicity, only one listener per message type is supported.
            this.fallbackDispatchEvent_ = function (t, msg) {
                var listener = _this.listeners_[t];
                if (listener) {
                    listener(msg);
                }
            };
            // Fallback implementation of |on|.
            this.fallbackOn_ = function (t, f) {
                _this.listeners_[t] = f;
            };
            // Shuts down the TCP server and peerconnection if they haven't already
            // shut down, fulfilling once both have terminated. Since neither
            // object's close() methods should ever reject, this should never reject.
            // TODO: close all sessions before fulfilling
            this.stopResources_ = function () {
                log.debug('freeing resources');
                // PeerConnection.close() returns void, implying that the shutdown is
                // effectively immediate.  However, we wrap it in a promise to ensure
                // that any exception is sent to the Promise.catch, rather than
                // propagating synchronously up the stack.
                return Promise.all([
                    new Promise(function (F, R) {
                        _this.peerConnection_.close();
                        F();
                    }),
                    _this.tcpServer_.shutdown()
                ]).then(function (discard) {
                });
            };
            // Invoked when a SOCKS client establishes a connection with the TCP server.
            // Note that Session closes the TCP connection and datachannel on any error.
            this.makeTcpToRtcSession_ = function (tcpConnection) {
                _this.pool_.openDataChannel().then(function (channel) {
                    var channelLabel = channel.getLabel();
                    log.info('associating channel %1 with new SOCKS client', channelLabel);
                    var session = new Session();
                    session.start(tcpConnection, channel, _this.bytesSentToPeer_, _this.bytesReceivedFromPeer_).catch(function (e) {
                        log.warn('session %1 failed to connect to remote endpoint: %2', [
                            channelLabel,
                            e.message
                        ]);
                    });
                    session.onceStopped.then(function () {
                        log.info('discarded session %1', channelLabel);
                    }, function (e) {
                        log.error('session %1 terminated with error: %2', [
                            channelLabel,
                            e.message
                        ]);
                    });
                }, function (e) {
                    log.error('failed to open channel for new SOCKS client: %1 ', e.message);
                    // TODO: return bytes to the client!
                });
            };
            this.handleSignalFromPeer = function (signal) {
                _this.peerConnection_.handleSignalMessage(signal);
            };
            this.toString = function () {
                var ret;
                var sessionsAsStrings = [];
                var label;
                ret = JSON.stringify({ tcpServer_: _this.tcpServer_.toString() });
                return ret;
            };
            if (!this.dispatchEvent_) {
                // CONSIDER: Remove this code once all users of this class move to
                // freedom.  See https://github.com/uProxy/uproxy/issues/733 for
                // possible solutions.
                this.listeners_ = {};
                this.on = this.fallbackOn_;
                this.dispatchEvent_ = this.fallbackDispatchEvent_;
            }
        }
        return SocksToRtc;
    })();
    _SocksToRtc.SocksToRtc = SocksToRtc; // class SocksToRtc
    // A Socks sesson links a Tcp connection to a particular data channel on the
    // peer connection. CONSIDER: when we have a lightweight webrtc provider, we
    // can use the DataChannel class directly here instead of the awkward pairing
    // of peerConnection with chanelLabel.
    var Session = (function () {
        function Session() {
            var _this = this;
            this.onceStopping_ = new Promise(function (F, R) {
                _this.fulfillStopping_ = F;
            });
            // The supplied TCP connection and datachannel must already be
            // successfully established.
            // Returns onceReady.
            this.start = function (tcpConnection, dataChannel, bytesSentToPeer, bytesReceivedFromPeer) {
                _this.tcpConnection_ = tcpConnection;
                _this.dataChannel_ = dataChannel;
                _this.bytesSentToPeer_ = bytesSentToPeer;
                _this.bytesReceivedFromPeer_ = bytesReceivedFromPeer;
                // The session is ready once we've completed both
                // auth and request handshakes.
                _this.onceReady = _this.doAuthHandshake_().then(_this.doRequestHandshake_).then(function (response) {
                    if (response.reply !== 0 /* SUCCEEDED */) {
                        throw new Error('handshake failed with reply code ' + socks.Reply[response.reply]);
                    }
                    log.info('%1: connected to remote host', [_this.longId()]);
                    log.debug('%1: remote peer bound address: %2', [
                        _this.longId(),
                        JSON.stringify(response.endpoint)
                    ]);
                });
                // Once the handshakes have completed, start forwarding data between the
                // socket and channel and listen for socket and channel termination.
                // If handshake fails, shutdown.
                _this.onceReady.then(function () {
                    _this.linkSocketAndChannel_();
                    // Shutdown once the data channel terminates.
                    _this.dataChannel_.onceClosed.then(function () {
                        if (_this.dataChannel_.dataFromPeerQueue.getLength() > 0) {
                            log.warn('%1: channel closed with %2 unprocessed incoming messages', _this.longId(), _this.dataChannel_.dataFromPeerQueue.getLength());
                        }
                        else {
                            log.info('%1: channel closed', _this.longId());
                        }
                        _this.fulfillStopping_();
                    });
                }, _this.fulfillStopping_);
                // Once shutdown has been requested, free resources.
                // TODO: This promise is only used for logging, so we should probably
                // remove it, or at least make it private.
                _this.onceStopped = _this.onceStopping_.then(_this.stopResources_);
                return _this.onceReady;
            };
            this.longId = function () {
                return 'session ' + _this.channelLabel() + ' (socket ' + _this.tcpConnection_.connectionId + ' ' + (_this.tcpConnection_.isClosed() ? 'closed' : 'open') + ')';
            };
            // Initiates shutdown of the TCP server and peerconnection.
            // Returns onceStopped.
            this.stop = function () {
                log.debug('%1: stop requested', [_this.longId()]);
                _this.fulfillStopping_();
                return _this.onceStopped;
            };
            // Closes the TCP connection and datachannel if they haven't already
            // closed, fulfilling once both have closed. Since neither object's
            // close() methods should ever reject, this should never reject.
            this.stopResources_ = function () {
                log.debug('%1: freeing resources', [_this.longId()]);
                // DataChannel.close() returns void, implying that it is
                // effectively immediate.  However, we wrap it in a promise to ensure
                // that any exception is sent to the Promise.catch, rather than
                // propagating synchronously up the stack.
                return Promise.all([
                    new Promise(function (F, R) {
                        _this.dataChannel_.close();
                        F();
                    }),
                    _this.tcpConnection_.close()
                ]).then(function (discard) {
                });
            };
            this.channelLabel = function () {
                return _this.dataChannel_.getLabel();
            };
            this.toString = function () {
                return JSON.stringify({
                    channelLabel_: _this.channelLabel(),
                    tcpConnection: _this.tcpConnection_.toString()
                });
            };
            // Receive a socks connection and send the initial Auth messages.
            // Assumes: no packet fragmentation.
            // TODO: send failure to client if auth fails
            // TODO: handle packet fragmentation:
            //   https://github.com/uProxy/uproxy/issues/323
            // TODO: Needs unit tests badly since it's mocked by several other tests.
            this.doAuthHandshake_ = function () {
                return _this.tcpConnection_.receiveNext().then(socks.interpretAuthHandshakeBuffer).then(function (auths) {
                    _this.tcpConnection_.send(socks.composeAuthResponse(0 /* NOAUTH */));
                });
            };
            // Handles the SOCKS handshake, fulfilling with the socks.Response instance
            // sent to the SOCKS client iff all the following steps succeed:
            //  - reads the next packet from the socket
            //  - parses this packet as a socks.Request instance
            //  - pauses the socket to avoid receiving data before it can be forwarded
            //  - forwards this to RtcToNet
            //  - receives the next message from the channel
            //  - parses this message as a socks.Response instance
            //  - forwards the socks.Response to the SOCKS client
            // If a response is not received from RtcToNet or any other error
            // occurs then we send a generic FAILURE response back to the SOCKS
            // client before rejecting.
            // TODO: Needs unit tests badly since it's mocked by several other tests.
            this.doRequestHandshake_ = function () {
                return _this.tcpConnection_.receiveNext().then(socks.interpretRequestBuffer).then(function (request) {
                    // The domain name is very sensitive, so we keep it out of the
                    // info-level logs, which may be uploaded.
                    log.debug('%1: received endpoint from SOCKS client: %2', [
                        _this.longId(),
                        JSON.stringify(request.endpoint)
                    ]);
                    _this.tcpConnection_.pause();
                    return _this.dataChannel_.send({ str: JSON.stringify(request) });
                }).then(function () {
                    // Equivalent to channel.receiveNext(), if it existed.
                    return new Promise(function (F, R) {
                        _this.dataChannel_.dataFromPeerQueue.setSyncNextHandler(F).catch(R);
                    });
                }).then(function (data) {
                    if (!data.str) {
                        throw new Error('received non-string data from peer ' + 'during handshake: ' + JSON.stringify(data));
                    }
                    try {
                        var response = JSON.parse(data.str);
                        if (!socks.isValidResponse(response)) {
                            throw new Error('invalid response received from peer ' + 'during handshake: ' + data.str);
                        }
                        return response;
                    }
                    catch (e) {
                        throw new Error('could not parse response from peer: ' + e.message);
                    }
                }).catch(function (e) {
                    log.debug('%1: unexpected failure during handshake, ' + 'returning generic FAILURE to SOCKS client: %2', [
                        _this.longId(),
                        e.message
                    ]);
                    return {
                        reply: 1 /* FAILURE */
                    };
                }).then(function (response) {
                    return _this.tcpConnection_.send(socks.composeResponseBuffer(response)).then(function (discard) {
                        return response;
                    });
                });
            };
            // Sends a packet over the data channel.
            // Invoked when a packet is received over the TCP socket.
            this.sendOnChannel_ = function (data) {
                return _this.dataChannel_.send({ buffer: data });
            };
            // Sends a packet over the TCP socket.
            // Invoked when a packet is received over the data channel.
            this.sendOnSocket_ = function (data) {
                if (!data.buffer) {
                    return Promise.reject(new Error('received non-buffer data from datachannel'));
                }
                _this.bytesReceivedFromPeer_.handle(data.buffer.byteLength);
                return _this.tcpConnection_.send(data.buffer);
            };
            // Configures forwarding of data from the TCP socket over the data channel
            // and vice versa. Should only be called once both socket and channel have
            // been successfully established.
            this.linkSocketAndChannel_ = function () {
                log.info('%1: linking socket and channel', _this.longId());
                var socketReader = function (data) {
                    _this.sendOnChannel_(data).then(function () {
                        _this.bytesSentToPeer_.handle(data.byteLength);
                    }, function (e) {
                        log.error('%1: failed to send data on datachannel: %2', _this.longId(), e.message);
                    });
                };
                _this.tcpConnection_.dataFromSocketQueue.setSyncHandler(socketReader);
                // Shutdown the session once the TCP connection terminates.
                // This should be safe now because
                // (1) this.tcpConnection_.dataFromPeerQueue has now been emptied into
                // this.dataChannel_.send() and (2) this.dataChannel_.close() should delay
                // closing until all pending messages have been sent.
                _this.tcpConnection_.onceClosed.then(function (kind) {
                    log.info('%1: socket closed (%2)', _this.longId(), tcp.SocketCloseKind[kind]);
                    _this.fulfillStopping_();
                });
                var channelReader = function (data) {
                    _this.sendOnSocket_(data).catch(function (e) {
                        // TODO: e is actually a freedom.Error (uproxy-lib 20+)
                        // errcode values are defined here:
                        //   https://github.com/freedomjs/freedom/blob/master/interface/core.tcpsocket.json
                        if (e.errcode === 'NOT_CONNECTED') {
                            // This can happen if, for example, there was still data to be
                            // read on the datachannel's queue when the socket closed.
                            log.warn('%1: tried to send data on closed socket: %2', [
                                _this.longId(),
                                e.errcode
                            ]);
                        }
                        else {
                            log.error('%1: failed to send data on socket: %2', [
                                _this.longId(),
                                e
                            ]);
                        }
                    });
                };
                _this.dataChannel_.dataFromPeerQueue.setSyncHandler(channelReader);
                // The TCP connection starts in the paused state.  However, in extreme
                // cases, enough data can arrive before the pause takes effect to put
                // the data channel into overflow.  In that case, the socket will
                // eventually be resumed by the overflow listener below.
                if (!_this.dataChannel_.isInOverflow()) {
                    _this.tcpConnection_.resume();
                }
                _this.dataChannel_.setOverflowListener(function (overflow) {
                    if (_this.tcpConnection_.isClosed()) {
                        return;
                    }
                    if (overflow) {
                        _this.tcpConnection_.pause();
                        log.debug('%1: Hit overflow, pausing socket', _this.longId());
                    }
                    else {
                        _this.tcpConnection_.resume();
                        log.debug('%1: Exited  overflow, resuming socket', _this.longId());
                    }
                });
            };
        }
        return Session;
    })();
    _SocksToRtc.Session = Session; // Session
})(SocksToRtc || (SocksToRtc = {})); // module SocksToRtc
module.exports = SocksToRtc;
