var arraybuffers = require('./arraybuffers');
// Small internal utility class for accumulating array buffers into a
// single buffer (until that buffer is big enough).
var Accumulator = (function () {
    function Accumulator(minSize, onBigEnoughBuffer) {
        var _this = this;
        this.onBigEnoughBuffer = onBigEnoughBuffer;
        // The buffers being acculated are stored in a list.
        this.buffers_ = [];
        // The current size of all accumulated buffers.
        this.currentSize_ = 0;
        // Set the minimal size. Returns true if it causes the buffer handler to be
        // called because the current size is bigger than the new minimal size.
        this.setMinSize = function (newMinSize) {
            _this.minSize_ = newMinSize;
            return _this.maybeBigEnoughNow();
        };
        // Throw the buffers away and start again from zero.
        this.clear = function () {
            _this.buffers_ = [];
            _this.currentSize_ = 0;
        };
        // Check if the buffer is big enough now and if so call the handler and
        // return the size of the handled accumulated buffer. Otherwise return zero.
        this.maybeBigEnoughNow = function () {
            if (_this.currentSize_ < _this.minSize_) {
                return 0;
            }
            // Big enough now, to clear out old stuff and call the handler!
            // Note: for the wild use case of doing something that ends up adding
            // more stuff to this array buffer accumulator, we use temporary
            // variables to avoid potential infinite loop of maybeBigEnoughNow being
            // called.
            var buffers = _this.buffers_;
            var accumulatedSize = _this.currentSize_;
            _this.clear();
            _this.onBigEnoughBuffer(arraybuffers.concat(buffers, accumulatedSize));
            return accumulatedSize;
        };
        // Add a buffer to the accumulated buffer(s). If this makes the accumulated
        // buffer bigger than the minimal size, then the handler is called and we
        // return the size of the buffer that was handled. Otherwise we return 0.
        this.addBuffer = function (a) {
            _this.buffers_.push(a);
            _this.currentSize_ += a.byteLength;
            return _this.maybeBigEnoughNow();
        };
        this.minSize_ = minSize;
    }
    return Accumulator;
})();
exports.Accumulator = Accumulator;
