// common-grunt-rules
/// <reference path='../../../third_party/typings/node/node.d.ts' />
var path = require('path');
var Rule = (function () {
    function Rule(config) {
        this.config = config;
    }
    // Note: argument is modified (and returned for conveniece);
    Rule.prototype.addCoverageToSpec = function (spec) {
        var basePath = path.dirname(spec.options.outfile);
        spec.options.template = require('grunt-template-jasmine-istanbul');
        spec.options.templateOptions = {
            files: ['**/*', '!node_modules/**'],
            // Output location for coverage results
            coverage: path.join(basePath, 'coverage/results.json'),
            report: [
                { type: 'html', options: { dir: path.join(basePath, 'coverage') } },
                { type: 'lcov', options: { dir: path.join(basePath, 'coverage') } }
            ]
        };
        return spec;
    };
    // Grunt Jasmine target creator
    // Assumes that the each spec file is a fully browserified js file.
    Rule.prototype.jasmineSpec = function (name, morefiles) {
        if (!morefiles) {
            morefiles = [];
        }
        return {
            src: [
                require.resolve('arraybuffer-slice'),
                require.resolve('es6-promise'),
                path.join(this.config.thirdPartyBuildPath, 'promise-polyfill.js'),
            ].concat(morefiles),
            options: {
                specs: [path.join(this.config.devBuildPath, name, '/**/*.spec.static.js')],
                outfile: path.join(this.config.devBuildPath, name, '/SpecRunner.html'),
                keepRunner: true
            }
        };
    };
    // Grunt browserify target creator
    Rule.prototype.browserify = function (filepath, options) {
        if (options === void 0) { options = {
            browserifyOptions: { standalone: 'browserified_exports' }
        }; }
        return {
            src: [path.join(this.config.devBuildPath, filepath + '.js')],
            dest: path.join(this.config.devBuildPath, filepath + '.static.js'),
            options: options
        };
    };
    // Note: argument is modified (and returned for conveniece);
    Rule.prototype.addCoverageToBrowserify = function (rule) {
        if (!rule.options.transform) {
            rule.options.transform = [];
        }
        rule.options.transform.push(['browserify-istanbul', { ignore: ['**/mocks/**', '**/*.spec.js'] }]);
        return rule;
    };
    // Grunt browserify target creator, instrumented for istanbul
    Rule.prototype.browserifySpec = function (filepath, options) {
        if (options === void 0) { options = {
            browserifyOptions: { standalone: 'browserified_exports' }
        }; }
        return {
            src: [path.join(this.config.devBuildPath, filepath + '.spec.js')],
            dest: path.join(this.config.devBuildPath, filepath + '.spec.static.js'),
            options: options
        };
    };
    // Copies libs from npm, local libraries, and third party libraries to the
    // destination folder.
    Rule.prototype.copyLibs = function (copyInfo) {
        var _this = this;
        // Default to empty list of dependencies.
        copyInfo.npmLibNames = copyInfo.npmLibNames || [];
        copyInfo.pathsFromDevBuild = copyInfo.pathsFromDevBuild || [];
        copyInfo.pathsFromThirdPartyBuild = copyInfo.pathsFromThirdPartyBuild || [];
        copyInfo.files = copyInfo.files || [];
        var destPath = path.join(this.config.devBuildPath, copyInfo.localDestPath);
        var destPathForLibs = path.join(destPath, this.config.localLibsDestPath);
        var allFilesForlibPaths = [];
        // The file-set for npm module files (or npm module output) from each of
        // |npmLibNames| to the destination path.
        copyInfo.npmLibNames.map(function (npmName) {
            var npmModuleDirName;
            if (path.dirname(npmName) === '.') {
                // Note: |path.dirname(npmName)| gives '.' when |npmName| is just the
                // npm module name.
                npmModuleDirName = npmName;
            }
            else {
                npmModuleDirName = path.dirname(npmName);
            }
            var absoluteNpmFilePath = require.resolve(npmName);
            allFilesForlibPaths.push({
                expand: false,
                nonull: true,
                src: [absoluteNpmFilePath],
                dest: path.join(destPath, npmModuleDirName, path.basename(absoluteNpmFilePath)),
                onlyIf: 'modified'
            });
        });
        // The file-set for all relevant files in pathsFromDevBuild.
        copyInfo.pathsFromDevBuild.map(function (libPath) {
            allFilesForlibPaths.push({
                expand: true,
                cwd: _this.config.devBuildPath,
                src: [
                    libPath + '/**/*',
                    '!' + libPath + '/**/*.ts',
                    '!' + libPath + '/**/*.spec.js',
                    '!' + libPath + '/**/SpecRunner.html'
                ],
                dest: destPathForLibs,
                onlyIf: 'modified'
            });
        });
        // Provide a file-set to be copied for each local third_party module that is
        // listed in |pathsFromthirdPartyBuild|.
        copyInfo.pathsFromThirdPartyBuild.map(function (libPath) {
            allFilesForlibPaths.push({
                expand: true,
                cwd: _this.config.thirdPartyBuildPath,
                src: [
                    libPath + '/**/*',
                    '!' + libPath + '/**/*.ts',
                    '!' + libPath + '/**/*.spec.js',
                    '!' + libPath + '/**/SpecRunner.html'
                ],
                dest: destPath,
                onlyIf: 'modified'
            });
        });
        return { files: allFilesForlibPaths.concat(copyInfo.files) };
    };
    return Rule;
})();
exports.Rule = Rule; // class Rule
