/// <reference path='../../../../third_party/freedom-typings/freedom-module-env.d.ts' />
/// <reference path='../../../../third_party/freedom-typings/rtcpeerconnection.d.ts' />
var logging = require('../../logging/logging');
var loggingTypes = require('../../loggingprovider/loggingprovider.types');
var peerconnection = require('../../webrtc/peerconnection');
exports.loggingController = freedom['loggingcontroller']();
exports.loggingController.setDefaultFilter(0 /* console */, 0 /* debug */);
exports.loggingController.setDefaultFilter(1 /* buffered */, 0 /* debug */);
exports.moduleName = 'freedom-chat';
exports.log = new logging.Log(exports.moduleName);
var parentFreedomModule = freedom();
function connectDataChannel(name, d) {
    d.onceOpened.then(function () {
        exports.log.info(name + ': onceOpened: ' + d.toString());
    });
    d.onceClosed.then(function () {
        exports.log.info(name + ': onceClosed: ' + d.toString());
    });
    d.dataFromPeerQueue.setSyncHandler(function (data) {
        exports.log.info('%1: dataFromPeer: %2', name, data);
        // Handle messages received on the datachannel(s).
        parentFreedomModule.emit('receive' + name, {
            message: data.str
        });
    });
    parentFreedomModule.on('send' + name, function (message) {
        d.send({ str: message.message });
    });
}
// Make a peer connection which logs stuff that happens.
function makePeerConnection(name) {
    var pcConfig = {
        iceServers: [{
            urls: ['stun:stun.l.google.com:19302']
        }, { urls: ['stun:stun1.l.google.com:19302'] }]
    };
    var pc = peerconnection.createPeerConnection(pcConfig, name);
    pc.onceConnecting.then(function () {
        exports.log.info(name + ': connecting...');
    });
    pc.onceConnected.then(function () {
        exports.log.info(name + ' connected');
    }, function (e) {
        exports.log.error('%1 failed to connect: %2', name, e.message);
    });
    pc.onceDisconnected.then(function () {
        exports.log.info(name + ': onceDisconnected');
    });
    pc.peerOpenedChannelQueue.setSyncHandler(function (d) {
        exports.log.info(name + ': peerOpenedChannelQueue: ' + d.toString());
        connectDataChannel(name, d);
    });
    return pc;
}
var a = makePeerConnection('A');
var b = makePeerConnection('B');
// Connect the two signalling channels. Normally, these messages would be sent
// over the internet.
a.signalForPeerQueue.setSyncHandler(function (message) {
    exports.log.info('a: sending signalling message to b.');
    b.handleSignalMessage(message);
});
b.signalForPeerQueue.setSyncHandler(function (message) {
    exports.log.info('b: sending signalling message to a.');
    a.handleSignalMessage(message);
});
// Negotiate a peerconnection. Once negotiated, enable the UI and add
// send/receive handlers.
a.negotiateConnection().then(function () {
    exports.log.info('a: negotiated connection');
}, function (e) {
    exports.log.error('could not negotiate peerconnection: ' + e.message);
    parentFreedomModule.emit('error', {});
}).then(function () {
    return a.openDataChannel('text');
}).then(function (aTextDataChannel) {
    connectDataChannel('A', aTextDataChannel);
    parentFreedomModule.emit('ready', {});
    parentFreedomModule.on('stop', function () {
        a.close();
    });
}).catch(function (e) {
    exports.log.error('error while opening datachannel: ' + e.message);
    parentFreedomModule.emit('error', {});
});
